import {Component, OnInit, OnChanges, ElementRef, Input, Output, EventEmitter, SimpleChanges} from '@angular/core';
import {Http, HTTP_PROVIDERS} from '@angular/http';
import 'rxjs/add/operator/map';

declare var bryntum: any;

@Component({
    selector: 'scheduler',
    template: '<div></div>',
    providers: [HTTP_PROVIDERS]
})
export class SchedulerComponent implements OnInit, OnChanges {

    private elementRef: ElementRef;
    private schedulerEngine: any;

    //Available features
    private featureInputs = [
        'columnLines', 'dependencies', 'eventDrag', 'eventContextMenu', 'eventDrag', 'eventDragCreate', 'eventEdit', 'eventFilter',
        'eventResize', 'eventTooltip', 'groupSummary', 'headerContextMenu', 'nonWorkingTime', 'scheduleTooltip',
        'summary', 'timeRanges', 'cellEdit', 'sort', 'group', 'stripe'
    ];

    //Available configs
    private configInputs = ['autoHeight', 'viewPreset', 'startDate', 'endDate', 'events', 'resources', 'columns', 'data', 'emptyText',
        'readOnly', 'rowHeight', 'barMargin'];

    // Configs
    @Input() url: string;

    @Input() autoHeight: boolean = true;
    @Input() columns: any[];
    @Input() events: any[];
    @Input() resources: any[];
    @Input() readOnly: boolean = false;
    @Input() rowHeight: number = 50;
    @Input() startDate: any;
    @Input() endDate: any;
    @Input() viewPreset: string = 'hourAndDay';
    @Input() barMargin: number = 5;

    // Features, only used for initialization
    @Input() columnLines: boolean | any = true;
    @Input() dependencies: boolean | any = false;
    @Input() eventDrag: boolean | any = true;
    @Input() eventContextMenu: boolean | any = true;
    @Input() eventDragCreator: boolean | any = true;
    @Input() cellEdit: boolean | any = true;
    @Input() eventEditor: boolean | any = true;
    @Input() eventFilter: boolean | any = true;
    @Input() eventResize: boolean | any = true;
    @Input() eventTooltip: boolean | any = true;
    @Input() groupSummary: boolean | any;
    @Input() headerContextMenu: boolean | any;
    @Input() labels: boolean | any;
    @Input() nonWorkingTime: boolean;
    @Input() scheduleTooltip: boolean | any = true;
    @Input() summaryToolbar: boolean | any;
    @Input() timeRanges: boolean | any[] = true;

    @Output() selectedEvent: string = '';
    @Output() onSchedulerEvents = new EventEmitter<any>();

    constructor(private http: Http, element: ElementRef) {
        // Needed later, used as target when rendering Bryntum Grid
        this.elementRef = element;
    }

    ngOnInit() {
        const
            // Features config object
            featureConfig = {},
            // Grid config object
            config = {
                // Render scheduler to components element
                appendTo: this.elementRef.nativeElement.firstElementChild,

                // Listeners, will relay events
                listeners: {
                    catchAll(event) {
                        if (event.type === 'eventselectionchange') {
                            this.selectedEvent = event.selected.length ? event.selected[0].name : '';
                        }

                        this.onSchedulerEvents.emit(event);
                    },

                    thisObj: this
                },

                features: featureConfig
            };

        // Pass configs on to scheduler
        this.configInputs.forEach(configName => {
            if (configName in this) {
                config[configName] = this[configName];
            }
        });

        // Pass feature configs on to scheduler
        this.featureInputs.forEach(featureName => {
            if (featureName in this) {
                featureConfig[featureName] = this[featureName];
            }
        });

        this.schedulerEngine = new bryntum.scheduler.Scheduler(config);

        // load data
        this.url && this.http.get(this.url)
            .map(r => r.json())
            .subscribe(
                data => this.loadData(data),
                err => console.error(err)
            );
    }

    /**
     * Consumes json in format { resources: [], events: [] }
     * @param data
     */
    loadData(data): void {
        const engine = this.schedulerEngine;
        engine.resources = data.resources.rows;
        engine.events = data.events.rows;
    }

    ngOnChanges(changes: SimpleChanges) {
        const me = this;

        if (me.schedulerEngine) {
            // Iterate over all changes
            Object.entries(changes).forEach(([name, {currentValue}]) => {
                // Apply changes that match configs to grid
                if (me.configInputs.includes(name)) {
                    me.schedulerEngine[name] = currentValue;
                }

                if (me.featureInputs.includes(name)) {
                    me.schedulerEngine[name] = currentValue;
                }
            })
        }
    }

    removeEvent() {
        const scheduler = this.schedulerEngine;
        scheduler.eventStore.remove(scheduler.selectedEvents);
        this.selectedEvent = '';
    }

    addEvent() {
        const scheduler = this.schedulerEngine,
            startDate = new Date(scheduler.startDate.getTime()),
            endDate = new Date(startDate.getTime());

        endDate.setHours(endDate.getHours() + 1);

        scheduler.eventStore.add({
            resourceId: scheduler.resourceStore.first.id,
            startDate: startDate,
            endDate: endDate,
            name: 'New task',
            eventType: 'Meeting'
        })
    }
}
