import {Component, OnInit, OnChanges, ElementRef, Input, Output, EventEmitter, SimpleChanges} from '@angular/core';
import {Scheduler} from 'scheduler';

@Component({
  selector: 'scheduler',
  template: '<div></div>'
})
export class SchedulerComponent implements OnInit, OnChanges {

  private elementRef: ElementRef;
  private schedulerEngine: Scheduler;

  //Available features
  private featureInputs = [
    'columnLines', 'dependencies', 'eventDrag', 'eventContextMenu', 'eventDrag', 'eventDragCreate', 'eventEdit', 'eventFilter',
    'eventResize', 'eventTooltip', 'groupSummary', 'headerContextMenu', 'nonWorkingTime', 'scheduleTooltip',
    'summary', 'timeRanges', 'cellEdit', 'sort', 'group', 'stripe'
  ];

  //Available configs
  private configInputs = ['autoHeight', 'viewPreset', 'startDate', 'endDate', 'events', 'resources', 'columns', 'data', 'emptyText',
    'readOnly', 'rowHeight', 'barMargin'];

  // Configs
  @Input() autoHeight: boolean = true;
  @Input() columns: object[];
  @Input() events: object[];
  @Input() resources: object[];
  @Input() readOnly: boolean = false;
  @Input() rowHeight: number = 50;
  @Input() startDate: any;
  @Input() endDate: any;
  @Input() viewPreset: string = 'hourAndDay';
  @Input() barMargin: number = 5;

  // Features, only used for initialization
  @Input() columnLines: boolean | object = true;
  @Input() dependencies: boolean | object = false;
  @Input() eventDrag: boolean | object = true;
  @Input() eventContextMenu: boolean | object = true;
  @Input() eventDragCreator: boolean | object = true;
  @Input() cellEdit: boolean | object = true;
  @Input() eventEditor: boolean | object = true;
  @Input() eventFilter: boolean | object = true;
  @Input() eventResize: boolean | object = true;
  @Input() eventTooltip: boolean | object = true;
  @Input() groupSummary: boolean | object;
  @Input() headerContextMenu: boolean | object;
  @Input() labels: boolean | object;
  @Input() nonWorkingTime: boolean;
  @Input() scheduleTooltip: boolean | object = true;
  @Input() summaryToolbar: boolean | object;
  @Input() timeRanges: boolean | object[] = true;

  @Output() selectedEvent: string = '';
  @Output() onSchedulerEvents = new EventEmitter<object>();

  constructor(element: ElementRef) {
    // Needed later, used as target when rendering Bryntum Grid
    this.elementRef = element;
  }

  ngOnInit() {
    const
      // Features config object
      featureConfig = {},
      // Grid config object
      config = {
        // Render scheduler to components element
        appendTo: this.elementRef.nativeElement.firstElementChild,

        // Listeners, will relay events
        listeners: {
          catchAll(event) {
            if (event.type === 'eventselectionchange') {
              this.selectedEvent = event.selected.length ? event.selected[0].name : '';
            }

            this.onSchedulerEvents.emit(event);
          },

          thisObj: this
        },

        features: featureConfig
      };

    // Pass configs on to scheduler
    this.configInputs.forEach(configName => {
      if (configName in this) {
        config[configName] = this[configName];
      }
    });

    // Pass feature configs on to scheduler
    this.featureInputs.forEach(featureName => {
      if (featureName in this) {
        featureConfig[featureName] = this[featureName];
      }
    });

    this.schedulerEngine = new Scheduler(config);
  }

  ngOnChanges(changes: SimpleChanges) {

    let me = this;

    if (me.schedulerEngine) {
      // Iterate over all changes
      Object.entries(changes).forEach(([name, {currentValue}]) => {
        // Apply changes that match configs to grid
        if (me.configInputs.includes(name)) {
          me.schedulerEngine[name] = currentValue;
        }

        if (me.featureInputs.includes(name)) {
          me.schedulerEngine[name] = currentValue;
        }
      })
    }
  }

  removeEvent() {
    const scheduler = this.schedulerEngine;
    scheduler.eventStore.remove(scheduler.selectedEvents);
    this.selectedEvent = '';
  }

  addEvent() {
    const scheduler = this.schedulerEngine,
      startDate = new Date(scheduler.startDate.getTime()),
      endDate = new Date(startDate.getTime());

    endDate.setHours(endDate.getHours() + 1);

    scheduler.eventStore.add({
      resourceId: scheduler.resourceStore.first.id,
      startDate: startDate,
      endDate: endDate,
      name: 'New task',
      eventType: 'Meeting'
    })
  }
}
